#include "avr_compiler.h"
#include "adc_driver.h"

// How many samples for each ADC channel. 
#define SAMPLE_COUNT 10 

// Sample storage (all four channels). 
uint8_t adcSamples[4][SAMPLE_COUNT];

uint16_t interrupt_count = 0;
uint8_t offset;

int main(void)
{
	// Move stored calibration values to ADC bB.  
	ADC_CalibrationValues_Set(&ADCB);

	// Get offset value for ADC B and shift value to match 8 bit resolution.  
 	offset = ADC_Offset_Get(&ADCB);

	// Set up ADC B to have signed conversion mode and 8 bit resolution.  
  	ADC_ConvMode_and_Resolution_Config(&ADCA, true, ADC_RESOLUTION_8BIT_gc);

	// Sample rate is CPUFREQ/16. Allow time for storing data.  
	ADC_Prescaler_Config(&ADCB, ADC_PRESCALER_DIV16_gc);

	// Set referance voltage on ADC B to be VCC-0.6 V. 
	ADC_Referance_Config(&ADCB, ADC_REFSEL_VCC_gc);

	// Setup channel 0, 1, 2 and 3 to have single ended input.  
	ADC_Ch_InputMode_and_Gain_Config(&ADCB.CH0,
	                                 ADC_CH_INPUTMODE_SINGLEENDED_gc,
	                                 ADC_CH_GAIN_1X_gc);

	//ADC_Ch_InputMode_and_Gain_Config(&ADCA.CH1,
	//                                 ADC_CH_INPUTMODE_SINGLEENDED_gc,
	//                                 ADC_CH_GAIN_1X_gc);

	//ADC_Ch_InputMode_and_Gain_Config(&ADCA.CH2,
	//                                 ADC_CH_INPUTMODE_SINGLEENDED_gc,
	//                                 ADC_CH_GAIN_1X_gc);

	//ADC_Ch_InputMode_and_Gain_Config(&ADCA.CH3,
	//                                 ADC_CH_INPUTMODE_SINGLEENDED_gc,
	//                                 ADC_CH_GAIN_1X_gc);

	// Set input to the channel in ADC B to be PIN 1  
	ADC_Ch_InputMux_Config(&ADCB.CH0, ADC_CH_MUXPOS_PIN1_gc, ADC_CH_MUXNEG_PIN0_gc);
	//ADC_Ch_InputMux_Config(&ADCA.CH1, ADC_CH_MUXPOS_PIN5_gc, ADC_CH_MUXNEG_PIN0_gc);
	//ADC_Ch_InputMux_Config(&ADCA.CH2, ADC_CH_MUXPOS_PIN6_gc, ADC_CH_MUXNEG_PIN0_gc);
	//ADC_Ch_InputMux_Config(&ADCA.CH3, ADC_CH_MUXPOS_PIN7_gc, ADC_CH_MUXNEG_PIN0_gc);

	// Setup sweep of all four virtual channels. 
	ADC_SweepChannels_Config(&ADCB, ADC_SWEEP_0_gc);

	// Enable low level interrupts on ADCB channel 0, on conversion complete.  
	ADC_Ch_Interrupts_Config(&ADCB.CH0, ADC_CH_INTMODE_COMPLETE_gc, ADC_CH_INTLVL_LO_gc);

	// Enable PMIC interrupt level low.  
	PMIC.CTRL |= PMIC_LOLVLEX_bm;

	// Enable global interrupts.  
	sei();

	// Enable ADC B with free running mode, 1.2V reference and unsigned conversion. 
	ADC_Enable(&ADCB);

	// Wait until common mode voltage is stable. Default clk is 2MHz and
	// therefore below the maximum frequency to use this function.  
	ADC_Wait_8MHz(&ADCB);

	// Enable free running mode.  
	ADC_FreeRunning_Enable(&ADCB);

	// Eternal loop to wait for the conversions and interrupts to finnish.  
	while(true){
	}
}


/*! Interrupt routine that reads out the result form the conversion on all
 *  channels to a global array. If the number of conversions carried out is less
 *  than the number given in the define SAMPLE_COUNT a new conversion on all the
 *  channels is started. If not the interrupt on ADC A channel 3 is disabled.
 *
 */

ISR(ADCB_CH1_vect)
{
	//  Read samples and clear interrupt flags.  
  	adcSamples[0][interrupt_count] = ADC_ResultCh_GetLowByte(&ADCB.CH0, offset);
	//adcSamples[1][interrupt_count] = ADC_ResultCh_GetLowByte(&ADCA.CH1, offset);
	//adcSamples[2][interrupt_count] = ADC_ResultCh_GetLowByte(&ADCA.CH2, offset);
	//adcSamples[3][interrupt_count] = ADC_ResultCh_GetLowByte(&ADCA.CH3, offset);


  	if(interrupt_count == SAMPLE_COUNT-1){
		ADC_Ch_Interrupts_Config(&ADCB.CH0, ADC_CH_INTMODE_COMPLETE_gc, ADC_CH_INTLVL_OFF_gc);
		ADC_FreeRunning_Disable(&ADCB);
		ADC_Disable(&ADCB);
	}
	interrupt_count ++;
}
